@@ -20,11 +20,10 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.RandomAccessFile;
-import java.net.URL;
 import java.security.KeyStore;
 import java.security.KeyStoreException;
-import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.UnrecoverableKeyException;
 import java.security.cert.CertificateException;
@@ -79,13 +78,13 @@ public class CreateVisibleSignature extends CreateSignatureBase
      * @param filename
      * @param x position of the signature field
      * @param y position of the signature field
-     * @param zoomPercent
-     * @param imageStream
+     * @param zoomPercent increase (positive value) or decrease (negative value) image with x percent.
+     * @param imageStream input stream of an image.
      * @param page the signature should be placed on
      * @throws IOException
      */
     public void setVisibleSignDesigner(String filename, int x, int y, int zoomPercent, 
-            FileInputStream imageStream, int page) 
+            InputStream imageStream, int page) 
             throws IOException
     {
         visibleSignDesigner = new PDVisibleSignDesigner(filename, imageStream, page);
@@ -95,11 +94,11 @@ public class CreateVisibleSignature extends CreateSignatureBase
     /**
      * Set visible signature designer for an existing signature field.
      * 
-     * @param zoomPercent
-     * @param imageStream
+     * @param zoomPercent increase (positive value) or decrease (negative value) image with x percent.
+     * @param imageStream input stream of an image.
      * @throws IOException
      */
-    public void setVisibleSignDesigner(int zoomPercent, FileInputStream imageStream) 
+    public void setVisibleSignDesigner(int zoomPercent, InputStream imageStream) 
             throws IOException
     {
         visibleSignDesigner = new PDVisibleSignDesigner(imageStream);
@@ -115,10 +114,9 @@ public class CreateVisibleSignature extends CreateSignatureBase
      * @param preferredSize
      * @param page
      * @param visualSignEnabled
-     * @throws IOException
      */
     public void setVisibleSignatureProperties(String name, String location, String reason, int preferredSize, 
-            int page, boolean visualSignEnabled) throws IOException
+            int page, boolean visualSignEnabled)
     {
         visibleSignatureProperties.signerName(name).signerLocation(location).signatureReason(reason).
                 preferredSize(preferredSize).page(page).visualSignEnabled(visualSignEnabled).
@@ -132,10 +130,9 @@ public class CreateVisibleSignature extends CreateSignatureBase
      * @param location
      * @param reason
      * @param visualSignEnabled
-     * @throws IOException
      */
     public void setVisibleSignatureProperties(String name, String location, String reason,
-            boolean visualSignEnabled) throws IOException
+            boolean visualSignEnabled)
     {
         visibleSignatureProperties.signerName(name).signerLocation(location).signatureReason(reason).
                 visualSignEnabled(visualSignEnabled).setPdVisibleSignature(visibleSignDesigner);
@@ -164,12 +161,12 @@ public class CreateVisibleSignature extends CreateSignatureBase
      *
      * @param inputFile The source pdf document file.
      * @param signedFile The file to be signed.
-     * @param tsaClient optional TSA client
+     * @param tsaUrl optional TSA url
      * @throws IOException
      */
-    public void signPDF(File inputFile, File signedFile, TSAClient tsaClient) throws IOException
+    public void signPDF(File inputFile, File signedFile, String tsaUrl) throws IOException
     {
-        this.signPDF(inputFile, signedFile, tsaClient, null);
+        this.signPDF(inputFile, signedFile, tsaUrl, null);
     }
 
     /**
@@ -177,26 +174,26 @@ public class CreateVisibleSignature extends CreateSignatureBase
      *
      * @param inputFile The source pdf document file.
      * @param signedFile The file to be signed.
-     * @param tsaClient optional TSA client
+     * @param tsaUrl optional TSA url
      * @param signatureFieldName optional name of an existing (unsigned) signature field
      * @throws IOException
      */
-    public void signPDF(File inputFile, File signedFile, TSAClient tsaClient, String signatureFieldName) throws IOException
+    public void signPDF(File inputFile, File signedFile, String tsaUrl, String signatureFieldName) throws IOException
     {
-        setTsaClient(tsaClient);
-
         if (inputFile == null || !inputFile.exists())
         {
             throw new IOException("Document for signing does not exist");
         }
 
+        setTsaUrl(tsaUrl);
+
         // creating output document and prepare the IO streams.
         FileOutputStream fos = new FileOutputStream(signedFile);
 
         // load document
         PDDocument doc = PDDocument.load(inputFile);
 
-        int accessPermissions = getMDPPermission(doc);
+        int accessPermissions = SigUtils.getMDPPermission(doc);
         if (accessPermissions == 1)
         {
             throw new IllegalStateException("No changes to the document are permitted due to DocMDP transform parameters dictionary");
@@ -222,7 +219,7 @@ public class CreateVisibleSignature extends CreateSignatureBase
         // PDF/A-1b requires PDF version 1.4 max, so don't increase the version on such files.
         if (doc.getVersion() >= 1.5f && accessPermissions == 0)
         {
-            setMDPPermission(doc, signature, 2);
+            SigUtils.setMDPPermission(doc, signature, 2);
         }
 
         PDAcroForm acroForm = doc.getDocumentCatalog().getAcroForm();
@@ -413,19 +410,11 @@ public class CreateVisibleSignature extends CreateSignatureBase
         char[] pin = args[1].toCharArray();
         keystore.load(new FileInputStream(ksFile), pin);
 
-        // TSA client
-        TSAClient tsaClient = null;
-        if (tsaUrl != null)
-        {
-            MessageDigest digest = MessageDigest.getInstance("SHA-256");
-            tsaClient = new TSAClient(new URL(tsaUrl), null, null, digest);
-        }
-
         File documentFile = new File(args[2]);
 
         CreateVisibleSignature signing = new CreateVisibleSignature(keystore, pin.clone());
 
-        FileInputStream imageStream = new FileInputStream(args[3]);
+        InputStream imageStream = new FileInputStream(args[3]);
 
         String name = documentFile.getName();
         String substring = name.substring(0, name.lastIndexOf('.'));
@@ -437,7 +426,7 @@ public class CreateVisibleSignature extends CreateSignatureBase
         imageStream.close();
         signing.setVisibleSignatureProperties("name", "location", "Security", 0, page, true);
         signing.setExternalSigning(externalSig);
-        signing.signPDF(documentFile, signedDocumentFile, tsaClient);
+        signing.signPDF(documentFile, signedDocumentFile, tsaUrl);
     }
 
     /**
