@@ -76,7 +76,7 @@ public class CMapParser
      * Parses a predefined CMap.
      *
      * @param name CMap name.
-     * @return The parsed predefined CMap as a java object.
+     * @return The parsed predefined CMap as a java object, never null.
      * @throws IOException If the CMap could not be parsed.
      */
     public CMap parsePredefined(String name) throws IOException
@@ -100,7 +100,7 @@ public class CMapParser
      * This will parse the stream and create a cmap object.
      *
      * @param input The CMAP stream to parse.
-     * @return The parsed stream as a java object.
+     * @return The parsed stream as a java object, never null.
      * @throws IOException If there is an error parsing the stream.
      */
     public CMap parse(InputStream input) throws IOException
@@ -302,7 +302,15 @@ public class CMapParser
             int mappedCode = (Integer) parseNextToken(cmapStream);
             if (startCode.length <= 2 && endCode.length <= 2)
             {
-                result.addCIDRange((char) start, (char) end, mappedCode);
+                // some CMaps are using CID ranges to map single values
+                if (end == start)
+                {
+                    result.addCIDMapping(mappedCode, start);
+                }
+                else
+                {
+                    result.addCIDRange((char) start, (char) end, mappedCode);
+                }
             }
             else
             {
