@@ -643,8 +643,8 @@ public class COSDictionary extends COSBase implements COSUpdateInfo
     }
 
     /**
-     * This is a convenience method that will get the dictionary object that is expected to be a name and convert it to
-     * a string. Null is returned if the entry does not exist in the dictionary.
+     * This is a convenience method that will get the dictionary object that is expected to be a
+     * string. Null is returned if the entry does not exist in the dictionary.
      *
      * @param key The key to the item in the dictionary.
      * @return The name converted to a string.
@@ -655,8 +655,8 @@ public class COSDictionary extends COSBase implements COSUpdateInfo
     }
 
     /**
-     * This is a convenience method that will get the dictionary object that is expected to be a name and convert it to
-     * a string. Null is returned if the entry does not exist in the dictionary.
+     * This is a convenience method that will get the dictionary object that is expected to be a
+     * string. Null is returned if the entry does not exist in the dictionary.
      *
      * @param key The key to the item in the dictionary.
      * @return The name converted to a string.
@@ -673,8 +673,8 @@ public class COSDictionary extends COSBase implements COSUpdateInfo
     }
 
     /**
-     * This is a convenience method that will get the dictionary object that is expected to be a name and convert it to
-     * a string. Null is returned if the entry does not exist in the dictionary.
+     * This is a convenience method that will get the dictionary object that is expected to be a
+     * string. Null is returned if the entry does not exist in the dictionary.
      *
      * @param key The key to the item in the dictionary.
      * @param defaultValue The default value to return.
@@ -686,8 +686,8 @@ public class COSDictionary extends COSBase implements COSUpdateInfo
     }
 
     /**
-     * This is a convenience method that will get the dictionary object that is expected to be a name and convert it to
-     * a string. Null is returned if the entry does not exist in the dictionary.
+     * This is a convenience method that will get the dictionary object that is expected to be a
+     * string. Null is returned if the entry does not exist in the dictionary.
      *
      * @param key The key to the item in the dictionary.
      * @param defaultValue The default value to return.
@@ -755,10 +755,10 @@ public class COSDictionary extends COSBase implements COSUpdateInfo
     public String getEmbeddedString(String embedded, COSName key, String defaultValue)
     {
         String retval = defaultValue;
-        COSDictionary dic = (COSDictionary) getDictionaryObject(embedded);
-        if (dic != null)
+        COSBase base = getDictionaryObject(embedded);
+        if (base instanceof COSDictionary)
         {
-            retval = dic.getString(key, defaultValue);
+            retval = ((COSDictionary) base).getString(key, defaultValue);
         }
         return retval;
     }
@@ -1281,6 +1281,26 @@ public class COSDictionary extends COSBase implements COSUpdateInfo
         return getItem(COSName.getPDFName(key));
     }
 
+    /**
+     * This is a special case of getItem that takes multiple keys, it will handle the situation
+     * where multiple keys could get the same value, ie if either CS or ColorSpace is used to get
+     * the colorspace. This will get an object from this dictionary.
+     *
+     * @param firstKey The first key to try.
+     * @param secondKey The second key to try.
+     *
+     * @return The object that matches the key.
+     */
+    public COSBase getItem(COSName firstKey, COSName secondKey)
+    {
+        COSBase retval = getItem(firstKey);
+        if (retval == null && secondKey != null)
+        {
+            retval = getItem(secondKey);
+        }
+        return retval;
+    }
+
     /**
      * Returns the names of the entries in this dictionary. The returned set is in the order the entries were added to
      * the dictionary.
