@@ -116,6 +116,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName CCITTFAX_DECODE = new COSName("CCITTFaxDecode");
     public static final COSName CCITTFAX_DECODE_ABBREVIATION = new COSName("CCF");
     public static final COSName CENTER_WINDOW = new COSName("CenterWindow");
+    public static final COSName CERT = new COSName("Cert");
     public static final COSName CF = new COSName("CF");
     public static final COSName CFM = new COSName("CFM");
     //** Acro form field type for choice fields.
@@ -135,6 +136,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName CMAPNAME = new COSName("CMapName");
     public static final COSName CMYK = new COSName("CMYK");
     public static final COSName CO = new COSName("CO");
+    public static final COSName COLOR = new COSName("Color");
     public static final COSName COLOR_BURN = new COSName("ColorBurn");
     public static final COSName COLOR_DODGE = new COSName("ColorDodge");
     public static final COSName COLORANTS = new COSName("Colorants");
@@ -193,6 +195,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName DOC_CHECKSUM = new COSName("DocChecksum");
     public static final COSName DOC_TIME_STAMP = new COSName("DocTimeStamp");
     public static final COSName DOCMDP = new COSName("DocMDP");
+    public static final COSName DOCUMENT = new COSName("Document");
     public static final COSName DOMAIN = new COSName("Domain");
     public static final COSName DOS = new COSName("DOS");
     public static final COSName DP = new COSName("DP");
@@ -272,6 +275,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName HIDE_MENUBAR = new COSName("HideMenubar");
     public static final COSName HIDE_TOOLBAR = new COSName("HideToolbar");
     public static final COSName HIDE_WINDOWUI = new COSName("HideWindowUI");
+    public static final COSName HUE = new COSName("Hue");
     // I
     public static final COSName I = new COSName("I");
     public static final COSName IC = new COSName("IC");
@@ -292,6 +296,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName INTERPOLATE = new COSName("Interpolate");
     public static final COSName IT = new COSName("IT");
     public static final COSName ITALIC_ANGLE = new COSName("ItalicAngle");
+    public static final COSName ISSUER = new COSName("Issuer");
     // J
     public static final COSName JAVA_SCRIPT = new COSName("JavaScript");
     public static final COSName JBIG2_DECODE = new COSName("JBIG2Decode");
@@ -301,6 +306,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     // K
     public static final COSName K = new COSName("K");
     public static final COSName KEYWORDS = new COSName("Keywords");
+    public static final COSName KEY_USAGE = new COSName("KeyUsage");
     public static final COSName KIDS = new COSName("Kids");
     // L
     public static final COSName L = new COSName("L");
@@ -340,6 +346,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName MCID = new COSName("MCID");
     public static final COSName MDP = new COSName("MDP");
     public static final COSName MEDIA_BOX = new COSName("MediaBox");
+    public static final COSName MEASURE = new COSName("Measure");
     public static final COSName METADATA = new COSName("Metadata");
     public static final COSName MISSING_WIDTH = new COSName("MissingWidth");
     public static final COSName MIX = new COSName("Mix");
@@ -353,6 +360,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName NAME = new COSName("Name");
     public static final COSName NAMES = new COSName("Names");
     public static final COSName NEED_APPEARANCES = new COSName("NeedAppearances");
+    public static final COSName NEW_WINDOW = new COSName("NewWindow");
     public static final COSName NEXT = new COSName("Next");
     public static final COSName NM = new COSName("NM");
     public static final COSName NON_EFONT_NO_WARN = new COSName("NonEFontNoWarn");
@@ -369,6 +377,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName OCGS = new COSName("OCGs");
     public static final COSName OCPROPERTIES = new COSName("OCProperties");
     public static final COSName OE = new COSName("OE");
+    public static final COSName OID = new COSName("OID");
     
     /**
      * "OFF", to be used for OCGs, not for Acroform
@@ -457,6 +466,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     // S
     public static final COSName S = new COSName("S");
     public static final COSName SA = new COSName("SA");
+    public static final COSName SATURATION = new COSName("Saturation");
     public static final COSName SCREEN = new COSName("Screen");
     public static final COSName SE = new COSName("SE");
     public static final COSName SEPARATION = new COSName("Separation");
@@ -489,9 +499,11 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName SUB_FILTER = new COSName("SubFilter");
     public static final COSName SUBJ = new COSName("Subj");
     public static final COSName SUBJECT = new COSName("Subject");
+    public static final COSName SUBJECT_DN = new COSName("SubjectDN");
     public static final COSName SUBTYPE = new COSName("Subtype");
     public static final COSName SUPPLEMENT = new COSName("Supplement");
     public static final COSName SV = new COSName("SV");
+    public static final COSName SV_CERT = new COSName("SVCert");
     public static final COSName SW = new COSName("SW");
     public static final COSName SY = new COSName("Sy");
     public static final COSName SYNCHRONOUS = new COSName("Synchronous");
@@ -532,6 +544,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName UNIX = new COSName("Unix");
     public static final COSName URI = new COSName("URI");
     public static final COSName URL = new COSName("URL");
+    public static final COSName URL_TYPE = new COSName("URLType");
     // V
     public static final COSName V = new COSName("V");
     public static final COSName VERISIGN_PPKVS = new COSName("VeriSign.PPKVS");
@@ -542,6 +555,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public static final COSName VIEW_CLIP = new COSName("ViewClip");
     public static final COSName VIEWER_PREFERENCES = new COSName("ViewerPreferences");
     public static final COSName VOLUME = new COSName("Volume");
+    public static final COSName VP = new COSName("VP");
     // W
     public static final COSName W = new COSName("W");
     public static final COSName W2 = new COSName("W2");
@@ -682,7 +696,7 @@ public final class COSName extends COSBase implements Comparable<COSName>
     public void writePDF(OutputStream output) throws IOException
     {
         output.write('/');
-        byte[] bytes = getName().getBytes(Charsets.US_ASCII);
+        byte[] bytes = getName().getBytes(Charsets.UTF_8);
         for (byte b : bytes)
         {
             int current = b & 0xFF;
