@@ -32,6 +32,7 @@ import org.apache.pdfbox.cos.COSDictionary;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSObject;
 import org.apache.pdfbox.cos.COSStream;
+import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDPage;
 import org.apache.pdfbox.pdmodel.PDResources;
@@ -291,13 +292,9 @@ public class Overlay
         for (COSStream contentStream : contentStreams)
         {
             InputStream in = contentStream.createInputStream();
-            byte[] buf = new byte[2048];
-            int n;
-            while ((n = in.read(buf)) > 0)
-            {
-                out.write(buf, 0, n);
-            }
+            IOUtils.copy(in, out);
             out.flush();
+            in.close();
         }
         out.close();
         return concatStream;
@@ -602,7 +599,8 @@ public class Overlay
     /**
      * Sets the all pages overlay PDF.
      * 
-     * @param allPagesOverlayPDF the all pages overlay PDF
+     * @param allPagesOverlayPDF the all pages overlay PDF. This should not be a PDDocument that you
+     * created on the fly, it should be saved first, if it contains any fonts that are subset.
      */
     public void setAllPagesOverlayPDF(PDDocument allPagesOverlayPDF)
     {
