@@ -16,6 +16,10 @@
  */
 package org.apache.pdfbox.pdmodel;
 
+import java.awt.Point;
+import java.awt.image.DataBuffer;
+import java.awt.image.Raster;
+import java.awt.image.WritableRaster;
 import java.io.BufferedOutputStream;
 import java.io.Closeable;
 import java.io.File;
@@ -95,10 +99,18 @@ public class PDDocument implements Closeable
      */
     static
     {
-    	PDDeviceRGB.INSTANCE.toRGB(new float[]{1,1,1,1});
+    	try
+        {
+            WritableRaster raster = Raster.createBandedRaster(DataBuffer.TYPE_BYTE, 1, 1, 3, new Point(0, 0));
+            PDDeviceRGB.INSTANCE.toRGBImage(raster);
+        }
+        catch (IOException ex)
+        {
+            LOG.debug("voodoo error", ex);
+        }
+
         try
         {
-            //TODO remove this and deprecated COSNumber statics in 3.0
             COSNumber.get("0");
             COSNumber.get("1");
         }
@@ -666,21 +678,9 @@ public class PDDocument implements Closeable
     public PDPage importPage(PDPage page) throws IOException
     {
         PDPage importedPage = new PDPage(new COSDictionary(page.getCOSObject()), resourceCache);
-        InputStream in = null;
-        try
-        {
-            in = page.getContents();
-            if (in != null)
-            {
-                PDStream dest = new PDStream(this, in, COSName.FLATE_DECODE);
-                importedPage.setContents(dest);
-            }
-            addPage(importedPage);
-        }
-        catch (IOException e)
-        {
-            IOUtils.closeQuietly(in);
-        }
+        PDStream dest = new PDStream(this, page.getContents(), COSName.FLATE_DECODE);
+        importedPage.setContents(dest);
+        addPage(importedPage);
         importedPage.setCropBox(page.getCropBox());
         importedPage.setMediaBox(page.getMediaBox());
         importedPage.setRotation(page.getRotation());
@@ -833,7 +833,8 @@ public class PDDocument implements Closeable
     }
 
     /**
-     * This will return the last signature.
+     * This will return the last signature from the field tree. Note that this may not be the
+     * last in time when empty signature fields are created first but signed after other fields.
      * 
      * @return the last signature as <code>PDSignatureField</code>.
      * @throws IOException if no document catalog can be found.
@@ -1287,7 +1288,7 @@ public class PDDocument implements Closeable
      * Save the PDF as an incremental update. This is only possible if the PDF was loaded from a
      * file or a stream, not if the document was created in PDFBox itself.
      *
-     * @param output stream to write
+     * @param output stream to write. It should <i><b>not</b></i> point to the source file.
      * @throws IOException if the output could not be written
      * @throws IllegalStateException if the document was not loaded from a file or a stream.
      */
@@ -1343,11 +1344,11 @@ public class PDDocument implements Closeable
      * {@code PDDocument} instance and only AFTER {@link ExternalSigningSupport} instance is used.
      * </p>
      *
-     * @param output stream to write final PDF
+     * @param output stream to write final PDF. It should <i><b>not</b></i> point to the source file.
      * @return instance to be used for external signing and setting CMS signature
      * @throws IOException if the output could not be written
      * @throws IllegalStateException if the document was not loaded from a file or a stream or
-     * signature optionss were not set.
+     * signature options were not set.
      */
     public ExternalSigningSupport saveIncrementalForExternalSigning(OutputStream output) throws IOException
     {
@@ -1355,7 +1356,18 @@ public class PDDocument implements Closeable
         {
             throw new IllegalStateException("document was not loaded from a file or a stream");
         }
-        int[] byteRange = getLastSignatureDictionary().getByteRange();
+        // PDFBOX-3978: getLastSignatureDictionary() not helpful if signing into a template
+        // that is not the last signature. So give higher priority to signature with update flag.
+        PDSignature foundSignature = null;
+        for (PDSignature sig : getSignatureDictionaries())
+        {
+            foundSignature = sig;
+            if (sig.getCOSObject().isNeedToBeUpdated())
+            {
+                break;
+            }
+        }
+        int[] byteRange = foundSignature.getByteRange();
         if (!Arrays.equals(byteRange, RESERVE_BYTE_RANGE))
         {
             throw new IllegalStateException("signature reserve byte range has been changed "
@@ -1584,6 +1596,8 @@ public class PDDocument implements Closeable
 
     /**
      * Returns the resource cache associated with this document, or null if there is none.
+     * 
+     * @return the resource cache or null.
      */
     public ResourceCache getResourceCache()
     {
