@@ -40,7 +40,9 @@ import org.apache.pdfbox.pdmodel.common.PDMetadata;
 import org.apache.pdfbox.pdmodel.common.PDRectangle;
 import org.apache.pdfbox.pdmodel.common.PDStream;
 import org.apache.pdfbox.pdmodel.interactive.action.PDPageAdditionalActions;
+import org.apache.pdfbox.pdmodel.interactive.annotation.AnnotationFilter;
 import org.apache.pdfbox.pdmodel.interactive.annotation.PDAnnotation;
+import org.apache.pdfbox.pdmodel.interactive.measurement.PDViewportDictionary;
 import org.apache.pdfbox.pdmodel.interactive.pagenavigation.PDThreadBead;
 import org.apache.pdfbox.pdmodel.interactive.pagenavigation.PDTransition;
 import org.apache.pdfbox.util.Matrix;
@@ -139,6 +141,13 @@ public class PDPage implements COSObjectable, PDContentStream
         return streams.iterator();
     }
     
+    /**
+     * Returns the content stream(s) of this page as a single input stream.
+     *
+     * @return An InputStream, never null. Multiple content streams are concatenated and separated
+     * with a newline. An empty stream is returned if the page doesn't have any content stream.
+     * @throws IOException If the stream could not be read
+     */
     @Override
     public InputStream getContents() throws IOException
     {
@@ -169,6 +178,8 @@ public class PDPage implements COSObjectable, PDContentStream
 
     /**
      * Returns true if this page has contents.
+     * 
+     * @return true if the page has contents.
      */
     public boolean hasContents()
     {
@@ -192,13 +203,12 @@ public class PDPage implements COSObjectable, PDContentStream
     {
         if (pageResources == null)
         {
-            COSDictionary resources = (COSDictionary)
-                    PDPageTree.getInheritableAttribute(page, COSName.RESOURCES);
+            COSBase base = PDPageTree.getInheritableAttribute(page, COSName.RESOURCES);
 
             // note: it's an error for resources to not be present
-            if (resources != null)
+            if (base instanceof COSDictionary)
             {
-                pageResources = new PDResources(resources, resourceCache);
+                pageResources = new PDResources((COSDictionary) base, resourceCache);
             }
         }
         return pageResources;
@@ -256,17 +266,19 @@ public class PDPage implements COSObjectable, PDContentStream
     }
 
     /**
-     * A rectangle, expressed in default user space units, defining the boundaries of the physical
-     * medium on which the page is intended to be displayed or printed.
+     * A rectangle, expressed in default user space units, defining the boundaries of the physical medium on which the
+     * page is intended to be displayed or printed.
+     * 
+     * @return the media box.
      */
     public PDRectangle getMediaBox()
     {
         if (mediaBox == null)
         {
-            COSArray array = (COSArray) PDPageTree.getInheritableAttribute(page, COSName.MEDIA_BOX);
-            if (array != null)
+            COSBase base = PDPageTree.getInheritableAttribute(page, COSName.MEDIA_BOX);
+            if (base instanceof COSArray)
             {
-                mediaBox = new PDRectangle(array);
+                mediaBox = new PDRectangle((COSArray) base);
             }
         }
         if (mediaBox == null)
@@ -296,16 +308,17 @@ public class PDPage implements COSObjectable, PDContentStream
     }
 
     /**
-     * A rectangle, expressed in default user space units, defining the visible region of default
-     * user space. When the page is displayed or printed, its contents are to be clipped (cropped)
-     * to this rectangle.
+     * A rectangle, expressed in default user space units, defining the visible region of default user space. When the
+     * page is displayed or printed, its contents are to be clipped (cropped) to this rectangle.
+     * 
+     * @return the crop box.
      */
     public PDRectangle getCropBox()
     {
-        COSArray array = (COSArray) PDPageTree.getInheritableAttribute(page, COSName.CROP_BOX);
-        if (array != null)
+        COSBase base = PDPageTree.getInheritableAttribute(page, COSName.CROP_BOX);
+        if (base instanceof COSArray)
         {
-            return clipToMediaBox(new PDRectangle(array));
+            return clipToMediaBox(new PDRectangle((COSArray) base));
         }
         else
         {
@@ -339,17 +352,15 @@ public class PDPage implements COSObjectable, PDContentStream
      */
     public PDRectangle getBleedBox()
     {
-        PDRectangle retval;
-        COSArray array = (COSArray) page.getDictionaryObject(COSName.BLEED_BOX);
-        if (array != null)
+        COSBase base = page.getDictionaryObject(COSName.BLEED_BOX);
+        if (base instanceof COSArray)
         {
-            retval = clipToMediaBox(new PDRectangle(array));
+            return clipToMediaBox(new PDRectangle((COSArray) base));
         }
         else
         {
-            retval = getCropBox();
+            return getCropBox();
         }
-        return retval;
     }
 
     /**
@@ -377,17 +388,15 @@ public class PDPage implements COSObjectable, PDContentStream
      */
     public PDRectangle getTrimBox()
     {
-        PDRectangle retval;
-        COSArray array = (COSArray) page.getDictionaryObject(COSName.TRIM_BOX);
-        if (array != null)
+        COSBase base = page.getDictionaryObject(COSName.TRIM_BOX);
+        if (base instanceof COSArray)
         {
-            retval = clipToMediaBox(new PDRectangle(array));
+            return clipToMediaBox(new PDRectangle((COSArray) base));
         }
         else
         {
-            retval = getCropBox();
+            return getCropBox();
         }
-        return retval;
     }
 
     /**
@@ -416,17 +425,15 @@ public class PDPage implements COSObjectable, PDContentStream
      */
     public PDRectangle getArtBox()
     {
-        PDRectangle retval;
-        COSArray array = (COSArray) page.getDictionaryObject(COSName.ART_BOX);
-        if (array != null)
+        COSBase base = page.getDictionaryObject(COSName.ART_BOX);
+        if (base instanceof COSArray)
         {
-            retval = clipToMediaBox(new PDRectangle(array));
+            return clipToMediaBox(new PDRectangle((COSArray) base));
         }
         else
         {
-            retval = getCropBox();
+            return getCropBox();
         }
-        return retval;
     }
 
     /**
@@ -534,12 +541,12 @@ public class PDPage implements COSObjectable, PDContentStream
         List<PDThreadBead> pdObjects = new ArrayList<PDThreadBead>();
         for (int i = 0; i < beads.size(); i++)
         {
-            COSDictionary beadDic = (COSDictionary) beads.getObject(i);
+            COSBase base = beads.getObject(i);
             PDThreadBead bead = null;
             // in some cases the bead is null
-            if (beadDic != null)
+            if (base instanceof COSDictionary)
             {
-                bead = new PDThreadBead(beadDic);
+                bead = new PDThreadBead((COSDictionary) base);
             }
             pdObjects.add(bead);
         }
@@ -565,10 +572,10 @@ public class PDPage implements COSObjectable, PDContentStream
     public PDMetadata getMetadata()
     {
         PDMetadata retval = null;
-        COSStream stream = (COSStream) page.getDictionaryObject(COSName.METADATA);
-        if (stream != null)
+        COSBase base = page.getDictionaryObject(COSName.METADATA);
+        if (base instanceof COSStream)
         {
-            retval = new PDMetadata(stream);
+            retval = new PDMetadata((COSStream) base);
         }
         return retval;
     }
@@ -590,8 +597,13 @@ public class PDPage implements COSObjectable, PDContentStream
      */
     public PDPageAdditionalActions getActions()
     {
-        COSDictionary addAct = (COSDictionary) page.getDictionaryObject(COSName.AA);
-        if (addAct == null)
+        COSDictionary addAct;
+        COSBase base = page.getDictionaryObject(COSName.AA);
+        if (base instanceof COSDictionary)
+        {
+            addAct = (COSDictionary) base;
+        }
+        else
         {
             addAct = new COSDictionary();
             page.setItem(COSName.AA, addAct);
@@ -614,8 +626,8 @@ public class PDPage implements COSObjectable, PDContentStream
      */
     public PDTransition getTransition()
     {
-        COSDictionary transitionDictionary = (COSDictionary) page.getDictionaryObject(COSName.TRANS);
-        return transitionDictionary == null ? null : new PDTransition(transitionDictionary);
+        COSBase base = page.getDictionaryObject(COSName.TRANS);
+        return base instanceof COSDictionary ? new PDTransition((COSDictionary) base) : null;
     }
 
     /**
@@ -638,7 +650,7 @@ public class PDPage implements COSObjectable, PDContentStream
         page.setItem(COSName.TRANS, transition);
         page.setItem(COSName.DUR, new COSFloat(duration));
     }
-
+    
     /**
      * This will return a list of the annotations for this page.
      *
@@ -649,14 +661,31 @@ public class PDPage implements COSObjectable, PDContentStream
      */
     public List<PDAnnotation> getAnnotations() throws IOException
     {
-        COSArrayList<PDAnnotation> retval;
-        COSArray annots = (COSArray) page.getDictionaryObject(COSName.ANNOTS);
-        if (annots == null)
+        return getAnnotations(new AnnotationFilter()
         {
-            return new COSArrayList<PDAnnotation>(page, COSName.ANNOTS);
-        }
-        else
+            @Override
+            public boolean accept(PDAnnotation annotation)
+            {
+                return true;
+            }
+        });
+    }    
+
+    /**
+     * This will return a list of the annotations for this page.
+     *
+     * @param annotationFilter the annotation filter provided allowing to filter out specific annotations
+     * @return List of the PDAnnotation objects, never null. The returned list is backed by the
+     * annotations COSArray, so any adding or deleting in this list will change the document too.
+     * 
+     * @throws IOException If there is an error while creating the annotation list.
+     */
+    public List<PDAnnotation> getAnnotations(AnnotationFilter annotationFilter) throws IOException
+    {
+        COSBase base = page.getDictionaryObject(COSName.ANNOTS);
+        if (base instanceof COSArray)
         {
+            COSArray annots = (COSArray) base;
             List<PDAnnotation> actuals = new ArrayList<PDAnnotation>();
             for (int i = 0; i < annots.size(); i++)
             {
@@ -665,11 +694,15 @@ public class PDPage implements COSObjectable, PDContentStream
                 {
                     continue;
                 }
-                actuals.add(PDAnnotation.createAnnotation(item));
+                PDAnnotation createdAnnotation = PDAnnotation.createAnnotation(item);
+                if (annotationFilter.accept(createdAnnotation))
+                {
+                    actuals.add(createdAnnotation);
+                }
             }
-            retval = new COSArrayList<PDAnnotation>(actuals, annots);
+            return new COSArrayList<PDAnnotation>(actuals, annots);
         }
-        return retval;
+        return new COSArrayList<PDAnnotation>(page, COSName.ANNOTS);
     }
 
     /**
@@ -696,9 +729,60 @@ public class PDPage implements COSObjectable, PDContentStream
 
     /**
      * Returns the resource cache associated with this page, or null if there is none.
+     * 
+     * @return the resource cache associated to this page.
      */
     public ResourceCache getResourceCache()
     {
         return resourceCache;
     }
+
+    /**
+     * Get the viewports.
+     *
+     * @return a list of viewports or null if there is no /VP entry.
+     */
+    public List<PDViewportDictionary> getViewports()
+    {
+        COSBase base = page.getDictionaryObject(COSName.VP);
+        if (!(base instanceof COSArray))
+        {
+            return null;
+        }
+        COSArray array = (COSArray) base;
+        List<PDViewportDictionary> viewports = new ArrayList<PDViewportDictionary>();
+        for (int i = 0; i < array.size(); ++i)
+        {
+            COSBase base2 = array.getObject(i);
+            if (base2 instanceof COSDictionary)
+            {
+                viewports.add(new PDViewportDictionary((COSDictionary) base2));
+            }
+            else
+            {
+                LOG.warn("Array element " + base2 + " is skipped, must be a (viewport) dictionary");
+            }
+        }
+        return viewports;
+    }
+
+    /**
+     * Set the viewports.
+     *
+     * @param viewports A list of viewports, or null if the entry is to be deleted.
+     */
+    public void setViewports(List<PDViewportDictionary> viewports)
+    {
+        if (viewports == null)
+        {
+            page.removeItem(COSName.VP);
+            return;
+        }
+        COSArray array = new COSArray();
+        for (PDViewportDictionary viewport : viewports)
+        {
+            array.add(viewport);
+        }
+        page.setItem(COSName.VP, array);
+    }
 }
