@@ -32,6 +32,7 @@ import org.apache.pdfbox.cos.COSInputStream;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSNull;
 import org.apache.pdfbox.cos.COSStream;
+import org.apache.pdfbox.filter.DecodeOptions;
 import org.apache.pdfbox.filter.Filter;
 import org.apache.pdfbox.filter.FilterFactory;
 import org.apache.pdfbox.io.IOUtils;
@@ -48,7 +49,7 @@ public class PDStream implements COSObjectable
     private final COSStream stream;
     
     /**
-     * Creates a new PDStream object.
+     * Creates a new empty PDStream object.
      * 
      * @param document The document that the stream will be part of.
      */
@@ -58,7 +59,7 @@ public class PDStream implements COSObjectable
     }
 
     /**
-     * Creates a new PDStream object.
+     * Creates a new empty PDStream object.
      *
      * @param document The document that the stream will be part of.
      */
@@ -235,6 +236,11 @@ public class PDStream implements COSObjectable
         return stream.createInputStream();
     }
 
+    public COSInputStream createInputStream(DecodeOptions options) throws IOException
+    {
+        return stream.createInputStream(options);
+    }
+
     /**
      * This will get a stream with some filters applied but not others. This is
      * useful when doing images, ie filters = [flate,dct], we want to remove
