@@ -63,7 +63,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
     */
     public static PDType0Font load(PDDocument doc, File file) throws IOException
     {
-        return new PDType0Font(doc, new TTFParser().parse(file), true, true);
+        return new PDType0Font(doc, new TTFParser().parse(file), true, true, false);
     }
 
     /**
@@ -76,7 +76,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
     */
     public static PDType0Font load(PDDocument doc, InputStream input) throws IOException
     {
-        return new PDType0Font(doc, new TTFParser().parse(input), true, true);
+        return new PDType0Font(doc, new TTFParser().parse(input), true, true, false);
     }
 
     /**
@@ -91,7 +91,7 @@ public class PDType0Font extends PDFont implements PDVectorFont
     public static PDType0Font load(PDDocument doc, InputStream input, boolean embedSubset)
             throws IOException
     {
-        return new PDType0Font(doc, new TTFParser().parse(input), embedSubset, true);
+        return new PDType0Font(doc, new TTFParser().parse(input), embedSubset, true, false);
     }
 
     /**
@@ -106,9 +106,65 @@ public class PDType0Font extends PDFont implements PDVectorFont
     public static PDType0Font load(PDDocument doc, TrueTypeFont ttf, boolean embedSubset)
             throws IOException
     {
-        return new PDType0Font(doc, ttf, embedSubset, false);
+        return new PDType0Font(doc, ttf, embedSubset, false, false);
     }
 
+    /**
+     * Loads a TTF to be embedded into a document as a vertical Type 0 font.
+     *
+     * @param doc The PDF document that will hold the embedded font.
+     * @param file A TrueType font.
+     * @return A Type0 font with a CIDFontType2 descendant.
+     * @throws IOException If there is an error reading the font file.
+     */
+    public static PDType0Font loadVertical(PDDocument doc, File file) throws IOException
+    {
+        return new PDType0Font(doc, new TTFParser().parse(file), true, true, true);
+    }
+
+    /**
+     * Loads a TTF to be embedded into a document as a vertical Type 0 font.
+     *
+     * @param doc The PDF document that will hold the embedded font.
+     * @param input A TrueType font.
+     * @return A Type0 font with a CIDFontType2 descendant.
+     * @throws IOException If there is an error reading the font stream.
+     */
+    public static PDType0Font loadVertical(PDDocument doc, InputStream input) throws IOException
+    {
+        return new PDType0Font(doc, new TTFParser().parse(input), true, true, true);
+    }
+
+    /**
+     * Loads a TTF to be embedded into a document as a vertical Type 0 font.
+     *
+     * @param doc The PDF document that will hold the embedded font.
+     * @param input A TrueType font.
+     * @param embedSubset True if the font will be subset before embedding
+     * @return A Type0 font with a CIDFontType2 descendant.
+     * @throws IOException If there is an error reading the font stream.
+     */
+    public static PDType0Font loadVertical(PDDocument doc, InputStream input, boolean embedSubset)
+            throws IOException
+    {
+        return new PDType0Font(doc, new TTFParser().parse(input), embedSubset, true, true);
+    }
+
+    /**
+     * Loads a TTF to be embedded into a document as a vertical Type 0 font.
+     *
+     * @param doc The PDF document that will hold the embedded font.
+     * @param ttf A TrueType font.
+     * @param embedSubset True if the font will be subset before embedding
+     * @return A Type0 font with a CIDFontType2 descendant.
+     * @throws IOException If there is an error reading the font stream.
+     */
+    public static PDType0Font loadVertical(PDDocument doc, TrueTypeFont ttf, boolean embedSubset)
+            throws IOException
+    {
+        return new PDType0Font(doc, ttf, embedSubset, false, true);
+    }
+    
     /**
      * Constructor for reading a Type0 font from a PDF file.
      * 
@@ -118,27 +174,34 @@ public class PDType0Font extends PDFont implements PDVectorFont
     public PDType0Font(COSDictionary fontDictionary) throws IOException
     {
         super(fontDictionary);
-        COSArray descendantFonts = (COSArray)dict.getDictionaryObject(COSName.DESCENDANT_FONTS);
-        COSDictionary descendantFontDictionary = (COSDictionary) descendantFonts.getObject(0);
-
-        if (descendantFontDictionary == null)
+        COSBase base = dict.getDictionaryObject(COSName.DESCENDANT_FONTS);
+        if (!(base instanceof COSArray))
+        {
+            throw new IOException("Missing descendant font array");
+        }
+        COSArray descendantFonts = (COSArray) base;
+        if (descendantFonts.size() == 0)
+        {
+            throw new IOException("Descendant font array is empty");
+        }
+        COSBase descendantFontDictBase = descendantFonts.getObject(0);
+        if (!(descendantFontDictBase instanceof COSDictionary))
         {
             throw new IOException("Missing descendant font dictionary");
         }
-
-        descendantFont = PDFontFactory.createDescendantFont(descendantFontDictionary, this);
+        descendantFont = PDFontFactory.createDescendantFont((COSDictionary) descendantFontDictBase, this);
         readEncoding();
         fetchCMapUCS2();
     }
 
-    /**
-    * Private. Creates a new TrueType font for embedding.
-    */
     private PDType0Font(PDDocument document, TrueTypeFont ttf, boolean embedSubset,
-            boolean closeOnSubset)
-            throws IOException
+            boolean closeOnSubset, boolean vertical) throws IOException
     {
-        embedder = new PDCIDFontType2Embedder(document, dict, ttf, embedSubset, this);
+        if (vertical)
+        {
+            ttf.enableVerticalSubstitutions();
+        }
+        embedder = new PDCIDFontType2Embedder(document, dict, ttf, embedSubset, this, vertical);
         descendantFont = embedder.getCIDFont();
         readEncoding();
         fetchCMapUCS2();
@@ -224,11 +287,11 @@ public class PDType0Font extends PDFont implements PDVectorFont
         PDCIDSystemInfo ros = descendantFont.getCIDSystemInfo();
         if (ros != null)
         {
-            isDescendantCJK = ros.getRegistry().equals("Adobe") &&
-                    (ros.getOrdering().equals("GB1") || 
-                     ros.getOrdering().equals("CNS1") ||
-                     ros.getOrdering().equals("Japan1") ||
-                     ros.getOrdering().equals("Korea1"));
+            isDescendantCJK = "Adobe".equals(ros.getRegistry()) &&
+                    ("GB1".equals(ros.getOrdering()) || 
+                     "CNS1".equals(ros.getOrdering()) ||
+                     "Japan1".equals(ros.getOrdering()) ||
+                     "Korea1".equals(ros.getOrdering()));
         }
     }
 
@@ -266,16 +329,9 @@ public class PDType0Font extends PDFont implements PDVectorFont
             // try to find the corresponding Unicode (UC2) CMap
             if (strName != null)
             {
-                CMap cMap = CMapManager.getPredefinedCMap(strName);
-                if (cMap != null)
-                {
-                    String ucs2Name = cMap.getRegistry() + "-" + cMap.getOrdering() + "-UCS2";
-                    CMap ucs2CMap = CMapManager.getPredefinedCMap(ucs2Name);
-                    if (ucs2CMap != null)
-                    {
-                        cMapUCS2 = ucs2CMap;
-                    }
-                }
+                CMap prdCMap = CMapManager.getPredefinedCMap(strName);
+                String ucs2Name = prdCMap.getRegistry() + "-" + prdCMap.getOrdering() + "-UCS2";
+                cMapUCS2 = CMapManager.getPredefinedCMap(ucs2Name);
             }
         }
     }
@@ -342,6 +398,12 @@ public class PDType0Font extends PDFont implements PDVectorFont
         return descendantFont.encode(unicode);
     }
 
+    @Override
+    public boolean hasExplicitWidth(int code) throws IOException
+    {
+        return descendantFont.hasExplicitWidth(code);
+    }
+
     @Override
     public float getAverageFontWidth()
     {
