@@ -17,8 +17,20 @@
 package org.apache.pdfbox.pdmodel.graphics.color;
 
 import java.awt.Color;
+import java.awt.Transparency;
+import java.awt.color.CMMException;
 import java.awt.color.ColorSpace;
+import java.awt.color.ICC_ColorSpace;
+import java.awt.color.ICC_Profile;
+import java.awt.color.ProfileDataException;
+import java.awt.image.BufferedImage;
+import java.awt.image.ComponentColorModel;
+import java.awt.image.DataBuffer;
+import java.awt.image.WritableRaster;
+import java.io.IOException;
+import java.io.InputStream;
 import java.util.Arrays;
+import java.util.List;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.cos.COSArray;
@@ -26,28 +38,15 @@ import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSFloat;
 import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.cos.COSStream;
-
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.common.COSArrayList;
 import org.apache.pdfbox.pdmodel.common.PDRange;
 import org.apache.pdfbox.pdmodel.common.PDStream;
-
-import java.awt.color.CMMException;
-import java.awt.color.ICC_ColorSpace;
-import java.awt.color.ICC_Profile;
-import java.awt.color.ProfileDataException;
-import java.awt.image.BufferedImage;
-
-import java.awt.image.WritableRaster;
-import java.io.InputStream;
-import java.io.IOException;
-
-import java.util.List;
 import org.apache.pdfbox.util.Charsets;
 
 /**
- * ICCBased colour spaces are based on a cross-platform colour profile as defined by the
+ * ICCBased color spaces are based on a cross-platform color profile as defined by the
  * International Color Consortium (ICC).
  *
  * @author Ben Litchfield
@@ -63,6 +62,7 @@ public final class PDICCBased extends PDCIEBasedColorSpace
     private PDColorSpace alternateColorSpace;
     private ICC_ColorSpace awtColorSpace;
     private PDColor initialColor;
+    private boolean isRGB = false;
 
     /**
      * Creates a new ICC color space with an empty stream.
@@ -131,11 +131,13 @@ public final class PDICCBased extends PDCIEBasedColorSpace
                 profile = ICC_Profile.getInstance(input);
                 if (is_sRGB(profile))
                 {
+                    isRGB = true;
                     awtColorSpace = (ICC_ColorSpace) ColorSpace.getInstance(ColorSpace.CS_sRGB);
                     iccProfile = awtColorSpace.getProfile();
                 }
                 else
                 {
+                    profile = ensureDisplayProfile(profile);
                     awtColorSpace = new ICC_ColorSpace(profile);
                     iccProfile = profile;
                 }
@@ -154,6 +156,9 @@ public final class PDICCBased extends PDCIEBasedColorSpace
                 awtColorSpace.toRGB(new float[awtColorSpace.getNumComponents()]);
                 // this one triggers an exception for PDFBOX-3549 with KCMS
                 new Color(awtColorSpace, new float[getNumberOfComponents()], 1f);
+                // PDFBOX-4015: this one triggers "CMMException: LCMS error 13" with LCMS
+                new ComponentColorModel(awtColorSpace, false, false, 
+                                        Transparency.OPAQUE, DataBuffer.TYPE_BYTE);
             }
         }
         catch (RuntimeException e)
@@ -166,7 +171,12 @@ public final class PDICCBased extends PDCIEBasedColorSpace
                 // fall back to alternateColorSpace color space
                 awtColorSpace = null;
                 alternateColorSpace = getAlternateColorSpace();
-                LOG.error("Can't read embedded ICC profile (" + e.getLocalizedMessage() + "), using alternate color space: " + alternateColorSpace.getName());
+                if (alternateColorSpace.equals(PDDeviceRGB.INSTANCE))
+                {
+                    isRGB = true;
+                }
+                LOG.warn("Can't read embedded ICC profile (" + e.getLocalizedMessage() + 
+                         "), using alternate color space: " + alternateColorSpace.getName());
                 initialColor = alternateColorSpace.getInitialColor();
             }
             else
@@ -191,13 +201,44 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         return deviceModel.equals("sRGB");
     }
 
+    // PDFBOX-4114: fix profile that has the wrong display class,
+    // as done by Harald Kuhr in twelvemonkeys JPEGImageReader.ensureDisplayProfile()
+    private static ICC_Profile ensureDisplayProfile(ICC_Profile profile)
+    {
+        if (profile.getProfileClass() != ICC_Profile.CLASS_DISPLAY)
+        {
+            byte[] profileData = profile.getData(); // Need to clone entire profile, due to a OpenJDK bug
+
+            if (profileData[ICC_Profile.icHdrRenderingIntent] == ICC_Profile.icPerceptual)
+            {
+                LOG.warn("ICC profile is Perceptual, ignoring, treating as Display class");
+            	intToBigEndian(ICC_Profile.icSigDisplayClass, profileData, ICC_Profile.icHdrDeviceClass);
+                return ICC_Profile.getInstance(profileData);
+            }
+        }
+        return profile;
+    }
+
+    private static void intToBigEndian(int value, byte[] array, int index)
+    {
+        array[index] = (byte) (value >> 24);
+        array[index + 1] = (byte) (value >> 16);
+        array[index + 2] = (byte) (value >> 8);
+        array[index + 3] = (byte) (value);
+    }
+
     @Override
     public float[] toRGB(float[] value) throws IOException
     {
+        if (isRGB)
+        {
+            return value;
+        }
         if (awtColorSpace != null)
         {
+            // PDFBOX-2142: clamp bad values
             // WARNING: toRGB is very slow when used with LUT-based ICC profiles
-            return awtColorSpace.toRGB(value);
+            return awtColorSpace.toRGB(clampColors(awtColorSpace, value));
         }
         else
         {
@@ -205,6 +246,18 @@ public final class PDICCBased extends PDCIEBasedColorSpace
         }
     }
 
+    private float[] clampColors(ICC_ColorSpace cs, float[] value)
+    {
+        float[] result = new float[value.length];
+        for (int i = 0; i < value.length; ++i)
+        {
+            float minValue = cs.getMinValue(i);
+            float maxValue = cs.getMaxValue(i);
+            result[i] = value[i] < minValue ? minValue : (value[i] > maxValue ? maxValue : value[i]);
+        }
+        return result;
+    }
+
     @Override
     public BufferedImage toRGBImage(WritableRaster raster) throws IOException
     {
@@ -332,9 +385,13 @@ public final class PDICCBased extends PDCIEBasedColorSpace
     }
 
     /**
-     * Returns the type of the color space in the ICC profile.
-     * Will be one of {@code TYPE_GRAY}, {@code TYPE_RGB}, or {@code TYPE_CMYK}.
-     * @return an ICC color space type
+     * Returns the type of the color space in the ICC profile. If the ICC profile is invalid, the
+     * type of the alternate colorspace is returned, which will be one of
+     * {@link ColorSpace#TYPE_GRAY TYPE_GRAY}, {@link ColorSpace#TYPE_RGB TYPE_RGB},
+     * {@link ColorSpace#TYPE_CMYK TYPE_CMYK}, or -1 if that one is invalid.
+     *
+     * @return an ICC color space type. See {@link ColorSpace#getType()} and the static values of
+     * {@link ColorSpace} for more details.
      */
     public int getColorSpaceType()
     {
