@@ -18,6 +18,7 @@ package org.apache.pdfbox.pdmodel.graphics.image;
 
 import java.awt.Graphics2D;
 import java.awt.Paint;
+import java.awt.Rectangle;
 import java.awt.RenderingHints;
 import java.awt.image.BufferedImage;
 import java.awt.image.WritableRaster;
@@ -37,7 +38,10 @@ import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
 import org.apache.pdfbox.cos.COSInputStream;
 import org.apache.pdfbox.cos.COSName;
+import org.apache.pdfbox.cos.COSObject;
 import org.apache.pdfbox.cos.COSStream;
+import org.apache.pdfbox.filter.DecodeOptions;
+import org.apache.pdfbox.filter.DecodeResult;
 import org.apache.pdfbox.io.IOUtils;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.PDResources;
@@ -65,6 +69,9 @@ public final class PDImageXObject extends PDXObject implements PDImage
     private SoftReference<BufferedImage> cachedImage;
     private PDColorSpace colorSpace;
 
+    // initialize to MAX_VALUE as we prefer lower subsampling when keeping/replacing cache.
+    private int cachedImageSubsampling = Integer.MAX_VALUE;
+
     /**
      * current resource dictionary (has color spaces)
      */
@@ -122,15 +129,24 @@ public final class PDImageXObject extends PDXObject implements PDImage
      */
     public PDImageXObject(PDStream stream, PDResources resources) throws IOException
     {
-        this(stream, resources, stream.createInputStream());
-    }
-    
-    // repairs parameters using decode result
-    private PDImageXObject(PDStream stream, PDResources resources, COSInputStream input)
-    {
-        super(repair(stream, input), COSName.IMAGE);
+        super(stream, COSName.IMAGE);
         this.resources = resources;
-        this.colorSpace = input.getDecodeResult().getJPXColorSpace();
+        List<COSName> filters = stream.getFilters();
+        if (filters != null && !filters.isEmpty() && COSName.JPX_DECODE.equals(filters.get(filters.size()-1)))
+        {
+            COSInputStream is = null;
+            try
+            {
+                is = stream.createInputStream();
+                DecodeResult decodeResult = is.getDecodeResult();
+                stream.getCOSObject().addAll(decodeResult.getParameters());
+                this.colorSpace = decodeResult.getJPXColorSpace();
+            }
+            finally
+            {
+                IOUtils.closeQuietly(is);
+            }
+        }
     }
 
     /**
@@ -335,13 +351,6 @@ public final class PDImageXObject extends PDXObject implements PDImage
         throw new IllegalArgumentException("Image type not supported: " + name);
     }
 
-    // repairs parameters using decode result
-    private static PDStream repair(PDStream stream, COSInputStream input)
-    {
-        stream.getCOSObject().addAll(input.getDecodeResult().getParameters());
-        return stream;
-    }
-
     /**
      * Returns the metadata associated with this XObject, or null if there is none.
      * @return the metadata associated with this object.
@@ -390,7 +399,16 @@ public final class PDImageXObject extends PDXObject implements PDImage
     @Override
     public BufferedImage getImage() throws IOException
     {
-        if (cachedImage != null)
+        return getImage(null, 1);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public BufferedImage getImage(Rectangle region, int subsampling) throws IOException
+    {
+        if (region == null && subsampling == cachedImageSubsampling && cachedImage != null)
         {
             BufferedImage cached = cachedImage.get();
             if (cached != null)
@@ -400,7 +418,7 @@ public final class PDImageXObject extends PDXObject implements PDImage
         }
 
         // get image as RGB
-        BufferedImage image = SampledImageReader.getRGBImage(this, getColorKeyMask());
+        BufferedImage image = SampledImageReader.getRGBImage(this, region, subsampling, getColorKeyMask());
 
         // soft mask (overrides explicit mask)
         PDImageXObject softMask = getSoftMask();
@@ -418,7 +436,14 @@ public final class PDImageXObject extends PDXObject implements PDImage
             }
         }
 
-        cachedImage = new SoftReference<BufferedImage>(image);
+        if (region == null && subsampling <= cachedImageSubsampling)
+        {
+            // only cache full-image renders, and prefer lower subsampling frequency, as lower
+            // subsampling means higher quality and longer render times.
+            cachedImageSubsampling = subsampling;
+            cachedImage = new SoftReference<BufferedImage>(image);
+        }
+
         return image;
     }
 
@@ -603,10 +628,27 @@ public final class PDImageXObject extends PDXObject implements PDImage
     {
         if (colorSpace == null)
         {
-            COSBase cosBase = getCOSObject().getDictionaryObject(COSName.COLORSPACE, COSName.CS);
+            COSBase cosBase = getCOSObject().getItem(COSName.COLORSPACE, COSName.CS);
             if (cosBase != null)
             {
+                COSObject indirect = null;
+                if (cosBase instanceof COSObject &&
+                        resources != null && resources.getResourceCache() != null)
+                {
+                    // PDFBOX-4022: use the resource cache because several images
+                    // might have the same colorspace indirect object.
+                    indirect = (COSObject) cosBase;
+                    colorSpace = resources.getResourceCache().getColorSpace(indirect);
+                    if (colorSpace != null)
+                    {
+                        return colorSpace;
+                    }
+                }
                 colorSpace = PDColorSpace.create(cosBase, resources);
+                if (indirect != null)
+                {
+                    resources.getResourceCache().put(indirect, colorSpace);
+                }
             }
             else if (isStencil())
             {
@@ -627,6 +669,12 @@ public final class PDImageXObject extends PDXObject implements PDImage
     {
         return getStream().createInputStream();
     }
+    
+    @Override
+    public InputStream createInputStream(DecodeOptions options) throws IOException
+    {
+        return getStream().createInputStream(options);
+    }
 
     @Override
     public InputStream createInputStream(List<String> stopFilters) throws IOException
