@@ -58,8 +58,8 @@ import org.apache.pdfbox.util.Matrix;
  */
 public final class PDAcroForm implements COSObjectable
 {
-	private static final Log LOG = LogFactory.getLog(PDAcroForm.class);
-	
+    private static final Log LOG = LogFactory.getLog(PDAcroForm.class);
+    
     private static final int FLAG_SIGNATURES_EXIST = 1;
     private static final int FLAG_APPEND_ONLY = 1 << 1;
 
@@ -103,10 +103,6 @@ public final class PDAcroForm implements COSObjectable
      */
     private void verifyOrCreateDefaults()
     {
-        // TODO: the handling of the missing properties is suitable
-        // if there are no entries at all. It might be necessary to enhance that
-        // if only parts are missing
-        
         final String adobeDefaultAppearanceString = "/Helv 0 Tf 0 g ";
 
         // DA entry is required
@@ -114,18 +110,26 @@ public final class PDAcroForm implements COSObjectable
         {
             setDefaultAppearance(adobeDefaultAppearanceString);
         }
-        
+
         // DR entry is required
-        if (getDefaultResources() == null)
+        PDResources defaultResources = getDefaultResources();
+        if (defaultResources == null)
         {
-            // Adobe Acrobat uses Helvetica as a default font and 
-            // stores that under the name '/Helv' in the resources dictionary
-            // Zapf Dingbats is included per default for check boxes and 
-            // radio buttons as /ZaDb.
-            PDResources resources = new PDResources();
-            resources.put(COSName.getPDFName("Helv"), PDType1Font.HELVETICA);
-            resources.put(COSName.getPDFName("ZaDb"), PDType1Font.ZAPF_DINGBATS);
-            setDefaultResources(resources);
+            defaultResources = new PDResources();
+            setDefaultResources(defaultResources);
+        }
+
+        // Adobe Acrobat uses Helvetica as a default font and 
+        // stores that under the name '/Helv' in the resources dictionary
+        // Zapf Dingbats is included per default for check boxes and 
+        // radio buttons as /ZaDb.
+        if (!defaultResources.getCOSObject().containsKey("Helv"))
+        {
+            defaultResources.put(COSName.getPDFName("Helv"), PDType1Font.HELVETICA);
+        }
+        if (!defaultResources.getCOSObject().containsKey("ZaDb"))
+        {
+            defaultResources.put(COSName.getPDFName("ZaDb"), PDType1Font.ZAPF_DINGBATS);
         }
     }
     
@@ -215,20 +219,20 @@ public final class PDAcroForm implements COSObjectable
      */
     public void flatten() throws IOException
     {
-    	// for dynamic XFA forms there is no flatten as this would mean to do a rendering
-    	// from the XFA content into a static PDF.
-    	if (xfaIsDynamic())
-    	{
-    		LOG.warn("Flatten for a dynamix XFA form is not supported");
-    		return;
-    	}
-    	
-    	List<PDField> fields = new ArrayList<PDField>();
-    	for (PDField field: getFieldTree())
-    	{
-    		fields.add(field);
-    	}
-    	flatten(fields, false);
+        // for dynamic XFA forms there is no flatten as this would mean to do a rendering
+        // from the XFA content into a static PDF.
+        if (xfaIsDynamic())
+        {
+            LOG.warn("Flatten for a dynamix XFA form is not supported");
+            return;
+        }
+        
+        List<PDField> fields = new ArrayList<PDField>();
+        for (PDField field: getFieldTree())
+        {
+            fields.add(field);
+        }
+        flatten(fields, false);
     }
     
     
@@ -247,20 +251,20 @@ public final class PDAcroForm implements COSObjectable
      */
     public void flatten(List<PDField> fields, boolean refreshAppearances) throws IOException
     {
-    	// for dynamic XFA forms there is no flatten as this would mean to do a rendering
-    	// from the XFA content into a static PDF.
-    	if (xfaIsDynamic())
-    	{
-    		LOG.warn("Flatten for a dynamix XFA form is not supported");
-    		return;
-    	}
-    	
-    	// refresh the appearances if set
-    	if (refreshAppearances)
-    	{
-    		refreshAppearances(fields);
-    	}
-    	
+        // for dynamic XFA forms there is no flatten as this would mean to do a rendering
+        // from the XFA content into a static PDF.
+        if (xfaIsDynamic())
+        {
+            LOG.warn("Flatten for a dynamix XFA form is not supported");
+            return;
+        }
+        
+        // refresh the appearances if set
+        if (refreshAppearances)
+        {
+            refreshAppearances(fields);
+        }
+        
         // indicates if the original content stream
         // has been wrapped in a q...Q pair.
         boolean isContentStreamWrapped;
@@ -363,10 +367,10 @@ public final class PDAcroForm implements COSObjectable
     {
         for (PDField field : getFieldTree())
         {
-        	if (field instanceof PDTerminalField)
-        	{
-        		((PDTerminalField) field).constructAppearances();
-        	}
+            if (field instanceof PDTerminalField)
+            {
+                ((PDTerminalField) field).constructAppearances();
+            }
         }
     }
 
@@ -381,10 +385,10 @@ public final class PDAcroForm implements COSObjectable
     {
         for (PDField field : fields)
         {
-        	if (field instanceof PDTerminalField)
-        	{
-        		((PDTerminalField) field).constructAppearances();
-        	}
+            if (field instanceof PDTerminalField)
+            {
+                ((PDTerminalField) field).constructAppearances();
+            }
         }
     }
     
@@ -503,7 +507,7 @@ public final class PDAcroForm implements COSObjectable
         // get the field from the field tree
         for (PDField field : getFieldTree())
         {
-        	if (field.getFullyQualifiedName().equals(fullyQualifiedName))
+            if (field.getFullyQualifiedName().equals(fullyQualifiedName))
             {
                 return field;
             }
@@ -627,13 +631,15 @@ public final class PDAcroForm implements COSObjectable
     }
     
     /**
-     * This will get the 'quadding' or justification of the text to be displayed.
+     * This will get the document-wide default value for the quadding/justification of variable text
+     * fields. 
+     * <p>
      * 0 - Left(default)<br>
      * 1 - Centered<br>
      * 2 - Right<br>
-     * Please see the QUADDING_CONSTANTS.
+     * See the QUADDING constants of {@link PDVariableText}.
      *
-     * @return The justification of the text strings.
+     * @return The justification of the variable text fields.
      */
     public int getQ()
     {
@@ -647,9 +653,10 @@ public final class PDAcroForm implements COSObjectable
     }
 
     /**
-     * This will set the quadding/justification of the text.  See QUADDING constants.
+     * This will set the document-wide default value for the quadding/justification of variable text
+     * fields. See the QUADDING constants of {@link PDVariableText}.
      *
-     * @param q The new text justification.
+     * @param q The justification of the variable text fields.
      */
     public void setQ(int q)
     {
@@ -697,22 +704,22 @@ public final class PDAcroForm implements COSObjectable
     }
     
     private Map<COSDictionary, Integer> buildAnnotationToPageRef() {
-    	Map<COSDictionary, Integer> annotationToPageRef = new HashMap<COSDictionary, Integer>();
-    	
-    	int idx = 0;
-    	for (PDPage page : document.getPages()) {
-    		try {
-				for (PDAnnotation annotation : page.getAnnotations()) {
-					if (annotation instanceof PDAnnotationWidget) {
-						annotationToPageRef.put(annotation.getCOSObject(), idx);
-					}
-				}
-			} catch (IOException e) {
-				LOG.warn("Can't retriev annotations for page " + idx);
-			}
-    		idx++;
-    	}    	
-    	return annotationToPageRef;
+        Map<COSDictionary, Integer> annotationToPageRef = new HashMap<COSDictionary, Integer>();
+        
+        int idx = 0;
+        for (PDPage page : document.getPages()) {
+            try {
+                for (PDAnnotation annotation : page.getAnnotations()) {
+                    if (annotation instanceof PDAnnotationWidget) {
+                        annotationToPageRef.put(annotation.getCOSObject(), idx);
+                    }
+                }
+            } catch (IOException e) {
+                LOG.warn("Can't retriev annotations for page " + idx);
+            }
+            idx++;
+        }        
+        return annotationToPageRef;
     }
     
     /**
@@ -770,7 +777,7 @@ public final class PDAcroForm implements COSObjectable
     private boolean resolveNeedsScaling(PDAppearanceStream appearanceStream)
     {
         // Check if there is a transformation within the XObjects content
-    	PDResources resources = appearanceStream.getResources();
+        PDResources resources = appearanceStream.getResources();
         return resources != null && resources.getXObjectNames().iterator().hasNext();
     }
 }
