@@ -1,6 +1,4 @@
 /*
- *  Copyright 2011 adam.
- * 
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -35,14 +33,23 @@ import org.apache.pdfbox.pdmodel.font.encoding.WinAnsiEncoding;
 import org.apache.pdfbox.rendering.PDFRenderer;
 import org.apache.pdfbox.text.PDFTextStripper;
 import org.junit.Assert;
+import org.junit.Before;
 import org.junit.Test;
 
 /**
  * 
  * @author adam
+ * @author Tilman Hausherr
  */
 public class PDFontTest
 {
+    private static final File OUT_DIR = new File("target/test-output");
+
+    @Before
+    public void setUp() throws Exception
+    {
+        OUT_DIR.mkdirs();
+    }
 
     /**
      * Test of the error reported in PDFBOX-988
@@ -133,6 +140,56 @@ public class PDFontTest
         ttf2.close();
     }
 
+    /**
+     * PDFBOX-4115: Test ability to create PDF with german umlaut glyphs with a type 1 font.
+     * Test for everything that went wrong before this was fixed.
+     *
+     * @throws IOException 
+     */
+    @Test
+    public void testPDFBOX4115() throws IOException
+    {
+        File fontFile = new File("target/fonts", "n019003l.pfb");
+        File outputFile = new File(OUT_DIR, "FontType1.pdf");
+        String text = "äöüÄÖÜ";
+
+        PDDocument doc = new PDDocument();
+
+        PDPage page = new PDPage();
+        PDPageContentStream contentStream = new PDPageContentStream(doc, page);
+
+        PDType1Font font = new PDType1Font(doc, new FileInputStream(fontFile), WinAnsiEncoding.INSTANCE);
+
+        contentStream.beginText();
+        contentStream.setFont(font, 10);
+        contentStream.newLineAtOffset(10, 700);
+        contentStream.showText(text);
+        contentStream.endText();
+        contentStream.close();
+
+        doc.addPage(page);
+
+        doc.save(outputFile);
+        doc.close();
+
+        doc = PDDocument.load(outputFile);
+
+        font = (PDType1Font) doc.getPage(0).getResources().getFont(COSName.getPDFName("F1"));
+        Assert.assertEquals(font.getEncoding(), WinAnsiEncoding.INSTANCE);
+
+        for (char c : text.toCharArray())
+        {
+            String name = font.getEncoding().getName(c);
+            Assert.assertEquals("dieresis", name.substring(1));
+            Assert.assertFalse(font.getPath(name).getBounds2D().isEmpty());
+        }
+
+        PDFTextStripper stripper = new PDFTextStripper();
+        Assert.assertEquals(text, stripper.getText(doc).trim());
+
+        doc.close();
+    }
+
     private void testPDFBox3826checkFonts(byte[] byteArray, File fontFile) throws IOException
     {
         PDDocument doc = PDDocument.load(byteArray);
@@ -157,8 +214,9 @@ public class PDFontTest
         new PDFRenderer(doc).renderImage(0);
 
         PDFTextStripper stripper = new PDFTextStripper();
+        stripper.setLineSeparator("\n");
         String text = stripper.getText(doc);
-        //Assert.assertEquals("testMultipleFontFileReuse1\r\ntestMultipleFontFileReuse2\r\ntestMultipleFontFileReuse3\r\n", text);
+        Assert.assertEquals("testMultipleFontFileReuse1\ntestMultipleFontFileReuse2\ntestMultipleFontFileReuse3", text.trim());
 
         doc.close();
     }
